#ifdef XP_WIN
#include <windows.h>
#endif
#include "zincPlugin.h"

NS_IMPL_ISUPPORTS3(zincPlugin, zincIPlugin, zincIPluginWindow, nsIClassInfo)

nsPluginInstanceBase*
NS_NewPluginInstance(nsPluginCreateData* aCreateDataStruct)
{
  zincPlugin* mgpi = new zincPlugin();
  NS_ADDREF(mgpi);
  return mgpi;
}

void
NS_DestroyPluginInstance(nsPluginInstanceBase* aPlugin)
{
  NS_STATIC_CAST(zincPlugin*, aPlugin)->shut();
  zincPlugin* gp = NS_STATIC_CAST(zincPlugin*, aPlugin);
  NS_RELEASE(gp);
}

NPError
NS_PluginInitialize()
{
  return NPERR_NO_ERROR;
}

void
NS_PluginShutdown()
{
}

char*
NPP_GetMIMEDescription()
{
  return "application/x-zinc-plugin::Mozilla General Plugin";
}


#ifdef XP_UNIX
NPError
NS_PluginGetValue(NPPVariable aVariable, void *aValue)
{
  if (aVariable == NPPVpluginNameString)
  {
    *((char **)aValue) = "Mozilla General Plugin";
    return NPERR_NO_ERROR;
  }
  else if (aVariable == NPPVpluginDescriptionString)
  {
    *((char**)aValue) = "A plugin that lets applications get a window handle.";
    return NPERR_NO_ERROR;
  }
  return NPERR_INVALID_PARAM;
}
#endif

zincPlugin::zincPlugin()
  : inited(FALSE)
{
}

NPBool
zincPlugin::init(NPWindow* aWindow)
{
  inited = TRUE;
  SetWindow(aWindow);
  return TRUE;
}

void
zincPlugin::shut()
{
  inited = FALSE;
  resizeMonitor = nsnull;
}

NPBool
zincPlugin::isInitialized()
{
  return inited;
}

NPError
zincPlugin::GetValue(NPPVariable variable, void *value)
{
  static nsIID scriptableIID = ZINCIPLUGIN_IID;
  nsIID* ptr;

  switch (variable)
  {
  case NPPVpluginNameString:
    *((char **)value) = "Mozilla General Plugin";
    return NPERR_NO_ERROR;
  case NPPVpluginDescriptionString:
    *((char **)value) = "Mozilla General Plugin";
    return NPERR_NO_ERROR;
  case NPPVpluginScriptableInstance:
    NS_ADDREF(this);
    *((nsISupports**)value) = NS_STATIC_CAST(zincIPlugin*, this);
    return NPERR_NO_ERROR;
  case NPPVpluginScriptableIID:
    ptr = (nsIID*)NPN_MemAlloc(sizeof(nsIID));
    if (ptr)
    {
      *ptr = scriptableIID;
      *(nsIID **)value = ptr;
    }
    else
      return NPERR_OUT_OF_MEMORY_ERROR;
    return NPERR_NO_ERROR;
  case NPPVpluginNeedsXEmbed:
    *(int*)value = 1;
    return NPERR_NO_ERROR;
  }
  return NPERR_INVALID_PARAM;
}

NPError
zincPlugin::SetWindow(NPWindow* aWindow)
{
  window = aWindow->window;
  width = aWindow->width;
  height = aWindow->height;

  if (resizeMonitor != nsnull)
  {
    resizeMonitor->OnResize(width, height);
  }
  return NPERR_NO_ERROR;
}


NS_IMETHODIMP
zincPlugin::GetWidth(PRUint32* owidth)
{
  *owidth = width;
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::GetHeight(PRUint32* oheight)
{
  *oheight = height;
  return NS_OK;
}


NS_IMETHODIMP
zincPlugin::GetFlags(PRUint32 *aFlags)
{
  *aFlags = nsIClassInfo::PLUGIN_OBJECT | nsIClassInfo::DOM_OBJECT;
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::GetImplementationLanguage(PRUint32 *aImplementationLanguage)
{
  *aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::GetInterfaces(PRUint32 *count, nsIID * **array)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetContractID(char * *aContractID)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetClassDescription(char * *aClassDescription)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetClassID(nsCID * *aClassID)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
zincPlugin::GetResizeMonitor(zincIPluginResizeMonitor** rm)
{
  *rm = resizeMonitor;
  if (*rm != nsnull)
    NS_ADDREF(*rm);
  return NS_OK;
}

NS_IMETHODIMP
zincPlugin::SetResizeMonitor(zincIPluginResizeMonitor* rm)
{
  resizeMonitor = rm;
  return NS_OK;
}

